#!/usr/bin/env python2

"""
Definition of exit codes for the main()
"""

SUCCESS = 0
ARGPARSE_FAILURE = 2
INVALID_INPUT = 10
REPO_PATH_INVALID = 11
REPO_PATH_NOT_REPO = 12
INVALID_VCS_LIMITS = 13
PROCESSING_FAILED = 20
UNHANDLED_EXCEPTION = 255
